@@ -4,13 +4,14 @@ from typing import Optional, Union
 import torch
 import torch.nn.functional as F
 
+from kornia.core import Tensor, concatenate, stack, tensor, zeros
 from kornia.geometry.conversions import angle_to_rotation_matrix, convert_points_from_homogeneous, rad2deg
 from kornia.geometry.linalg import transform_points
 from kornia.geometry.transform import pyrdown
 from kornia.testing import KORNIA_CHECK_LAF, KORNIA_CHECK_SHAPE
 
 
-def get_laf_scale(LAF: torch.Tensor) -> torch.Tensor:
+def get_laf_scale(LAF: Tensor) -> Tensor:
     """Return a scale of the LAFs.
 
     Args:
@@ -33,7 +34,7 @@ def get_laf_scale(LAF: torch.Tensor) -> torch.Tensor:
     return out.abs().sqrt()
 
 
-def get_laf_center(LAF: torch.Tensor) -> torch.Tensor:
+def get_laf_center(LAF: Tensor) -> Tensor:
     """Return a center (keypoint) of the LAFs.
 
     Args:
@@ -51,18 +52,18 @@ def get_laf_center(LAF: torch.Tensor) -> torch.Tensor:
         >>> output = get_laf_center(input)  # BxNx2
     """
     KORNIA_CHECK_LAF(LAF)
-    out: torch.Tensor = LAF[..., 2]
+    out = LAF[..., 2]
     return out
 
 
-def get_laf_orientation(LAF: torch.Tensor) -> torch.Tensor:
+def get_laf_orientation(LAF: Tensor) -> Tensor:
     """Return orientation of the LAFs, in degrees.
 
     Args:
-        LAF: (torch.Tensor): tensor [BxNx2x3].
+        LAF: (Tensor): tensor [BxNx2x3].
 
     Returns:
-        torch.Tensor: tensor  BxNx1 .
+        Tensor: tensor  BxNx1 .
 
     Shape:
         - Input: :math: `(B, N, 2, 3)`
@@ -73,11 +74,11 @@ def get_laf_orientation(LAF: torch.Tensor) -> torch.Tensor:
         >>> output = get_laf_orientation(input)  # BxNx1
     """
     KORNIA_CHECK_LAF(LAF)
-    angle_rad: torch.Tensor = torch.atan2(LAF[..., 0, 1], LAF[..., 0, 0])
+    angle_rad = torch.atan2(LAF[..., 0, 1], LAF[..., 0, 0])
     return rad2deg(angle_rad).unsqueeze(-1)
 
 
-def set_laf_orientation(LAF: torch.Tensor, angles_degrees: torch.Tensor) -> torch.Tensor:
+def set_laf_orientation(LAF: Tensor, angles_degrees: Tensor) -> Tensor:
     """Change the orientation of the LAFs.
 
     Args:
@@ -93,16 +94,14 @@ def set_laf_orientation(LAF: torch.Tensor, angles_degrees: torch.Tensor) -> torc
     """
     KORNIA_CHECK_LAF(LAF)
     B, N = LAF.shape[:2]
-    rotmat: torch.Tensor = angle_to_rotation_matrix(angles_degrees).view(B * N, 2, 2)
-    laf_out: torch.Tensor = torch.cat(
+    rotmat = angle_to_rotation_matrix(angles_degrees).view(B * N, 2, 2)
+    laf_out = concatenate(
         [torch.bmm(make_upright(LAF).view(B * N, 2, 3)[:, :2, :2], rotmat), LAF.view(B * N, 2, 3)[:, :2, 2:]], dim=2
     ).view(B, N, 2, 3)
     return laf_out
 
 
-def laf_from_center_scale_ori(
-    xy: torch.Tensor, scale: Optional[torch.Tensor] = None, ori: Optional[torch.Tensor] = None
-) -> torch.Tensor:
+def laf_from_center_scale_ori(xy: Tensor, scale: Optional[Tensor] = None, ori: Optional[Tensor] = None) -> Tensor:
     """Return orientation of the LAFs, in radians. Useful to create kornia LAFs from OpenCV keypoints.
 
     Args:
@@ -120,15 +119,15 @@ def laf_from_center_scale_ori(
     if scale is None:
         scale = torch.ones(B, N, 1, 1, device=device, dtype=dtype)
     if ori is None:
-        ori = torch.zeros(B, N, 1, device=device, dtype=dtype)
+        ori = zeros(B, N, 1, device=device, dtype=dtype)
     KORNIA_CHECK_SHAPE(scale, ["B", "N", "1", "1"])
     KORNIA_CHECK_SHAPE(ori, ["B", "N", "1"])
-    unscaled_laf: torch.Tensor = torch.cat([angle_to_rotation_matrix(ori.squeeze(-1)), xy.unsqueeze(-1)], dim=-1)
-    laf: torch.Tensor = scale_laf(unscaled_laf, scale)
+    unscaled_laf = concatenate([angle_to_rotation_matrix(ori.squeeze(-1)), xy.unsqueeze(-1)], dim=-1)
+    laf = scale_laf(unscaled_laf, scale)
     return laf
 
 
-def scale_laf(laf: torch.Tensor, scale_coef: Union[float, torch.Tensor]) -> torch.Tensor:
+def scale_laf(laf: Tensor, scale_coef: Union[float, Tensor]) -> Tensor:
     """Multiplies region part of LAF ([:, :, :2, :2]) by a scale_coefficient.
 
     So the center, shape and orientation of the local feature stays the same, but the region area changes.
@@ -150,14 +149,14 @@ def scale_laf(laf: torch.Tensor, scale_coef: Union[float, torch.Tensor]) -> torc
         >>> scale = 0.5
         >>> output = scale_laf(input, scale)  # BxNx2x3
     """
-    if (type(scale_coef) is not float) and (type(scale_coef) is not torch.Tensor):
-        raise TypeError("scale_coef should be float or torch.Tensor " "Got {}".format(type(scale_coef)))
+    if (type(scale_coef) is not float) and (type(scale_coef) is not Tensor):
+        raise TypeError("scale_coef should be float or Tensor " "Got {}".format(type(scale_coef)))
     KORNIA_CHECK_LAF(laf)
-    centerless_laf: torch.Tensor = laf[:, :, :2, :2]
-    return torch.cat([scale_coef * centerless_laf, laf[:, :, :, 2:]], dim=3)
+    centerless_laf = laf[:, :, :2, :2]
+    return concatenate([scale_coef * centerless_laf, laf[:, :, :, 2:]], dim=3)
 
 
-def make_upright(laf: torch.Tensor, eps: float = 1e-9) -> torch.Tensor:
+def make_upright(laf: Tensor, eps: float = 1e-9) -> Tensor:
     """Rectify the affine matrix, so that it becomes upright.
 
     Args:
@@ -181,19 +180,19 @@ def make_upright(laf: torch.Tensor, eps: float = 1e-9) -> torch.Tensor:
     # The function is equivalent to doing 2x2 SVD and resetting rotation
     # matrix to an identity: U, S, V = svd(LAF); LAF_upright = U * S.
     b2a2 = torch.sqrt(laf[..., 0:1, 1:2] ** 2 + laf[..., 0:1, 0:1] ** 2) + eps
-    laf1_ell = torch.cat([(b2a2 / det).contiguous(), torch.zeros_like(det)], dim=3)
-    laf2_ell = torch.cat(  # type: ignore
+    laf1_ell = concatenate([(b2a2 / det).contiguous(), torch.zeros_like(det)], dim=3)
+    laf2_ell = concatenate(
         [
             ((laf[..., 1:2, 1:2] * laf[..., 0:1, 1:2] + laf[..., 1:2, 0:1] * laf[..., 0:1, 0:1]) / (b2a2 * det)),
             (det / b2a2).contiguous(),
         ],
         dim=3,
     )
-    laf_unit_scale = torch.cat([torch.cat([laf1_ell, laf2_ell], dim=2), laf[..., :, 2:3]], dim=3)
+    laf_unit_scale = concatenate([concatenate([laf1_ell, laf2_ell], dim=2), laf[..., :, 2:3]], dim=3)
     return scale_laf(laf_unit_scale, scale)
 
 
-def ellipse_to_laf(ells: torch.Tensor) -> torch.Tensor:
+def ellipse_to_laf(ells: Tensor) -> Tensor:
     """Convert ellipse regions to LAF format.
 
     Ellipse (a, b, c) and upright covariance matrix [a11 a12; 0 a22] are connected
@@ -222,8 +221,8 @@ def ellipse_to_laf(ells: torch.Tensor) -> torch.Tensor:
     if dim != 5:
         raise TypeError("ellipse shape should be must be [BxNx5]. " "Got {}".format(ells.size()))
     # Previous implementation was incorrectly using Cholesky decomp as matrix sqrt
-    # ell_shape = torch.cat([torch.cat([ells[..., 2:3], ells[..., 3:4]], dim=2).unsqueeze(2),
-    #                       torch.cat([ells[..., 3:4], ells[..., 4:5]], dim=2).unsqueeze(2)], dim=2).view(-1, 2, 2)
+    # ell_shape = concatenate([concatenate([ells[..., 2:3], ells[..., 3:4]], dim=2).unsqueeze(2),
+    #                       concatenate([ells[..., 3:4], ells[..., 4:5]], dim=2).unsqueeze(2)], dim=2).view(-1, 2, 2)
     # out = torch.matrix_power(torch.cholesky(ell_shape, False), -1).view(B, N, 2, 2)
 
     # We will calculate 2x2 matrix square root via special case formula
@@ -237,12 +236,12 @@ def ellipse_to_laf(ells: torch.Tensor) -> torch.Tensor:
     a12 = torch.zeros_like(a11)
     a22 = ells[..., 4:5].abs().sqrt()
     a21 = ells[..., 3:4] / (a11 + a22).clamp(1e-9)
-    A = torch.stack([a11, a12, a21, a22], dim=-1).view(B, N, 2, 2).inverse()
-    out = torch.cat([A, ells[..., :2].view(B, N, 2, 1)], dim=3)
+    A = stack([a11, a12, a21, a22], dim=-1).view(B, N, 2, 2).inverse()
+    out = concatenate([A, ells[..., :2].view(B, N, 2, 1)], dim=3)
     return out
 
 
-def laf_to_boundary_points(LAF: torch.Tensor, n_pts: int = 50) -> torch.Tensor:
+def laf_to_boundary_points(LAF: Tensor, n_pts: int = 50) -> Tensor:
     """Convert LAFs to boundary points of the regions + center.
 
     Used for local features visualization, see visualize_laf function.
@@ -260,7 +259,7 @@ def laf_to_boundary_points(LAF: torch.Tensor, n_pts: int = 50) -> torch.Tensor:
     """
     KORNIA_CHECK_LAF(LAF)
     B, N, _, _ = LAF.size()
-    pts = torch.cat(
+    pts = concatenate(
         [
             torch.sin(torch.linspace(0, 2 * math.pi, n_pts - 1)).unsqueeze(-1),
             torch.cos(torch.linspace(0, 2 * math.pi, n_pts - 1)).unsqueeze(-1),
@@ -269,15 +268,15 @@ def laf_to_boundary_points(LAF: torch.Tensor, n_pts: int = 50) -> torch.Tensor:
         dim=1,
     )
     # Add origin to draw also the orientation
-    pts = torch.cat([torch.tensor([0.0, 0.0, 1.0]).view(1, 3), pts], dim=0).unsqueeze(0).expand(B * N, n_pts, 3)
+    pts = concatenate([tensor([0.0, 0.0, 1.0]).view(1, 3), pts], dim=0).unsqueeze(0).expand(B * N, n_pts, 3)
     pts = pts.to(LAF.device).to(LAF.dtype)
-    aux = torch.tensor([0.0, 0.0, 1.0]).view(1, 1, 3).expand(B * N, 1, 3)
-    HLAF = torch.cat([LAF.view(-1, 2, 3), aux.to(LAF.device).to(LAF.dtype)], dim=1)
+    aux = tensor([0.0, 0.0, 1.0]).view(1, 1, 3).expand(B * N, 1, 3)
+    HLAF = concatenate([LAF.view(-1, 2, 3), aux.to(LAF.device).to(LAF.dtype)], dim=1)
     pts_h = torch.bmm(HLAF, pts.permute(0, 2, 1)).permute(0, 2, 1)
     return convert_points_from_homogeneous(pts_h.view(B, N, n_pts, 3))
 
 
-def get_laf_pts_to_draw(LAF: torch.Tensor, img_idx: int = 0):
+def get_laf_pts_to_draw(LAF: Tensor, img_idx: int = 0):
     """Return numpy array for drawing LAFs (local features).
 
     Args:
@@ -305,7 +304,7 @@ def get_laf_pts_to_draw(LAF: torch.Tensor, img_idx: int = 0):
     return (pts_np[..., 0], pts_np[..., 1])
 
 
-def denormalize_laf(LAF: torch.Tensor, images: torch.Tensor) -> torch.Tensor:
+def denormalize_laf(LAF: Tensor, images: Tensor) -> Tensor:
     """De-normalize LAFs from scale to image scale.
 
         B,N,H,W = images.size()
@@ -338,7 +337,7 @@ def denormalize_laf(LAF: torch.Tensor, images: torch.Tensor) -> torch.Tensor:
     return coef.expand_as(LAF) * LAF
 
 
-def normalize_laf(LAF: torch.Tensor, images: torch.Tensor) -> torch.Tensor:
+def normalize_laf(LAF: Tensor, images: Tensor) -> Tensor:
     """Normalize LAFs to [0,1] scale from pixel scale. See below:
         B,N,H,W = images.size()
         MIN_SIZE = min(H,W)
@@ -349,11 +348,11 @@ def normalize_laf(LAF: torch.Tensor, images: torch.Tensor) -> torch.Tensor:
         [a21/MIN_SIZE a22/MIN_SIZE y/H]
 
     Args:
-        LAF: (torch.Tensor).
-        images: (torch.Tensor) images, LAFs are detected in
+        LAF: (Tensor).
+        images: (Tensor) images, LAFs are detected in
 
     Returns:
-        LAF: (torch.Tensor).
+        LAF: (Tensor).
 
     Shape:
         - Input: :math:`(B, N, 2, 3)`
@@ -361,8 +360,8 @@ def normalize_laf(LAF: torch.Tensor, images: torch.Tensor) -> torch.Tensor:
     """
     KORNIA_CHECK_LAF(LAF)
     _, _, h, w = images.size()
-    wf: float = float(w)
-    hf: float = float(h)
+    wf = float(w)
+    hf = float(h)
     min_size = min(hf, wf)
     coef = torch.ones(1, 1, 2, 3).to(LAF.dtype).to(LAF.device) / min_size
     coef[0, 0, 0, 2] = 1.0 / wf
@@ -370,7 +369,7 @@ def normalize_laf(LAF: torch.Tensor, images: torch.Tensor) -> torch.Tensor:
     return coef.expand_as(LAF) * LAF
 
 
-def generate_patch_grid_from_normalized_LAF(img: torch.Tensor, LAF: torch.Tensor, PS: int = 32) -> torch.Tensor:
+def generate_patch_grid_from_normalized_LAF(img: Tensor, LAF: Tensor, PS: int = 32) -> Tensor:
     """Helper function for affine grid generation.
 
     Args:
@@ -389,15 +388,15 @@ def generate_patch_grid_from_normalized_LAF(img: torch.Tensor, LAF: torch.Tensor
     # and extraction at arbitrary other
     LAF_renorm = denormalize_laf(LAF, img)
 
-    grid = F.affine_grid(LAF_renorm.view(B * N, 2, 3), [B * N, ch, PS, PS], align_corners=False)  # type: ignore
+    grid = F.affine_grid(LAF_renorm.view(B * N, 2, 3), [B * N, ch, PS, PS], align_corners=False)
     grid[..., :, 0] = 2.0 * grid[..., :, 0].clone() / float(w) - 1.0
     grid[..., :, 1] = 2.0 * grid[..., :, 1].clone() / float(h) - 1.0
     return grid
 
 
 def extract_patches_simple(
-    img: torch.Tensor, laf: torch.Tensor, PS: int = 32, normalize_lafs_before_extraction: bool = True
-) -> torch.Tensor:
+    img: Tensor, laf: Tensor, PS: int = 32, normalize_lafs_before_extraction: bool = True
+) -> Tensor:
     """Extract patches defined by LAFs from image tensor.
 
     No smoothing applied, huge aliasing (better use extract_patches_from_pyramid).
@@ -413,7 +412,7 @@ def extract_patches_simple(
     """
     KORNIA_CHECK_LAF(laf)
     if normalize_lafs_before_extraction:
-        nlaf: torch.Tensor = normalize_laf(laf, img)
+        nlaf = normalize_laf(laf, img)
     else:
         nlaf = laf
     _, ch, h, w = img.size()
@@ -424,18 +423,15 @@ def extract_patches_simple(
         grid = generate_patch_grid_from_normalized_LAF(img[i : i + 1], nlaf[i : i + 1], PS).to(img.device)
         out.append(
             F.grid_sample(
-                img[i : i + 1].expand(grid.size(0), ch, h, w),
-                grid,  # type: ignore
-                padding_mode="border",
-                align_corners=False,
+                img[i : i + 1].expand(grid.size(0), ch, h, w), grid, padding_mode="border", align_corners=False
             )
         )
-    return torch.cat(out, dim=0).view(B, N, ch, PS, PS)
+    return concatenate(out, dim=0).view(B, N, ch, PS, PS)
 
 
 def extract_patches_from_pyramid(
-    img: torch.Tensor, laf: torch.Tensor, PS: int = 32, normalize_lafs_before_extraction: bool = True
-) -> torch.Tensor:
+    img: Tensor, laf: Tensor, PS: int = 32, normalize_lafs_before_extraction: bool = True
+) -> Tensor:
     """Extract patches defined by LAFs from image tensor.
 
     Patches are extracted from appropriate pyramid level.
@@ -451,7 +447,7 @@ def extract_patches_from_pyramid(
     """
     KORNIA_CHECK_LAF(laf)
     if normalize_lafs_before_extraction:
-        nlaf: torch.Tensor = normalize_laf(laf, img)
+        nlaf = normalize_laf(laf, img)
     else:
         nlaf = laf
     B, N, _, _ = laf.size()
@@ -460,7 +456,7 @@ def extract_patches_from_pyramid(
     pyr_idx = scale.log2().relu().long()
     cur_img = img
     cur_pyr_level = 0
-    out = torch.zeros(B, N, ch, PS, PS).to(nlaf.dtype).to(nlaf.device)
+    out = zeros(B, N, ch, PS, PS).to(nlaf.dtype).to(nlaf.device)
     while min(cur_img.size(2), cur_img.size(3)) >= PS:
         _, ch, h, w = cur_img.size()
         # for loop temporarily, to be refactored
@@ -471,10 +467,7 @@ def extract_patches_from_pyramid(
             scale_mask = (scale_mask > 0).view(-1)
             grid = generate_patch_grid_from_normalized_LAF(cur_img[i : i + 1], nlaf[i : i + 1, scale_mask, :, :], PS)
             patches = F.grid_sample(
-                cur_img[i : i + 1].expand(grid.size(0), ch, h, w),
-                grid,  # type: ignore
-                padding_mode="border",
-                align_corners=False,
+                cur_img[i : i + 1].expand(grid.size(0), ch, h, w), grid, padding_mode="border", align_corners=False
             )
             out[i].masked_scatter_(scale_mask.view(-1, 1, 1, 1), patches)
         cur_img = pyrdown(cur_img)
@@ -482,7 +475,7 @@ def extract_patches_from_pyramid(
     return out
 
 
-def laf_is_inside_image(laf: torch.Tensor, images: torch.Tensor, border: int = 0) -> torch.Tensor:
+def laf_is_inside_image(laf: Tensor, images: Tensor, border: int = 0) -> Tensor:
     """Check if the LAF is touching or partly outside the image boundary.
 
     Returns the mask of LAFs, which are fully inside the image, i.e. valid.
@@ -497,15 +490,15 @@ def laf_is_inside_image(laf: torch.Tensor, images: torch.Tensor, border: int = 0
     """
     KORNIA_CHECK_LAF(laf)
     _, _, h, w = images.size()
-    pts: torch.Tensor = laf_to_boundary_points(laf, 12)
-    good_lafs_mask: torch.Tensor = (
+    pts = laf_to_boundary_points(laf, 12)
+    good_lafs_mask = (
         (pts[..., 0] >= border) * (pts[..., 0] <= w - border) * (pts[..., 1] >= border) * (pts[..., 1] <= h - border)
     )
     good_lafs_mask = good_lafs_mask.min(dim=2)[0]
     return good_lafs_mask
 
 
-def laf_to_three_points(laf: torch.Tensor):
+def laf_to_three_points(laf: Tensor):
     """Convert local affine frame(LAF) to alternative representation: coordinates of LAF center, LAF-x unit vector,
     LAF-y unit vector.
 
@@ -516,11 +509,11 @@ def laf_to_three_points(laf: torch.Tensor):
         threepts :math:`(B, N, 2, 3)`.
     """
     KORNIA_CHECK_LAF(laf)
-    three_pts: torch.Tensor = torch.stack([laf[..., 2] + laf[..., 0], laf[..., 2] + laf[..., 1], laf[..., 2]], dim=-1)
+    three_pts = stack([laf[..., 2] + laf[..., 0], laf[..., 2] + laf[..., 1], laf[..., 2]], dim=-1)
     return three_pts
 
 
-def laf_from_three_points(threepts: torch.Tensor):
+def laf_from_three_points(threepts: Tensor):
     """Convert three points to local affine frame.
 
     Order is (0,0), (0, 1), (1, 0).
@@ -531,13 +524,11 @@ def laf_from_three_points(threepts: torch.Tensor):
     Returns:
         laf :math:`(B, N, 2, 3)`.
     """
-    laf: torch.Tensor = torch.stack(
-        [threepts[..., 0] - threepts[..., 2], threepts[..., 1] - threepts[..., 2], threepts[..., 2]], dim=-1
-    )
+    laf = stack([threepts[..., 0] - threepts[..., 2], threepts[..., 1] - threepts[..., 2], threepts[..., 2]], dim=-1)
     return laf
 
 
-def perspective_transform_lafs(trans_01: torch.Tensor, lafs_1: torch.Tensor) -> torch.Tensor:
+def perspective_transform_lafs(trans_01: Tensor, lafs_1: Tensor) -> Tensor:
     r"""Function that applies perspective transformations to a set of local affine frames (LAFs).
 
     Args:
@@ -582,7 +573,7 @@ def perspective_transform_lafs(trans_01: torch.Tensor, lafs_1: torch.Tensor) ->
     """
     KORNIA_CHECK_LAF(lafs_1)
     if not torch.is_tensor(trans_01):
-        raise TypeError("Input type is not a torch.Tensor")
+        raise TypeError("Input type is not a Tensor")
 
     if not trans_01.device == lafs_1.device:
         raise TypeError("Tensor must be in the same device")
