@@ -1,13 +1,15 @@
-from typing import Callable, Dict, List, Tuple, Union
+from typing import Any, Dict, List, Tuple, Union
 
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 
 from kornia.constants import pi
+from kornia.core import Tensor, tensor, zeros
 from kornia.filters import GaussianBlur2d, SpatialGradient
 from kornia.geometry.conversions import cart2pol
 from kornia.utils import create_meshgrid
+from kornia.utils.helpers import map_location_to_cpu
 
 # Precomputed coefficients for Von Mises kernel, given N and K(appa).
 sqrt2: float = 1.4142135623730951
@@ -22,7 +24,7 @@ urls: Dict[str, str] = {
 }
 
 
-def get_grid_dict(patch_size: int = 32) -> Dict[str, torch.Tensor]:
+def get_grid_dict(patch_size: int = 32) -> Dict[str, Tensor]:
     r"""Get cartesian and polar parametrizations of grid."""
     kgrid = create_meshgrid(height=patch_size, width=patch_size, normalized_coordinates=True)
     x = kgrid[0, :, :, 0]
@@ -32,9 +34,9 @@ def get_grid_dict(patch_size: int = 32) -> Dict[str, torch.Tensor]:
     return grid_dict
 
 
-def get_kron_order(d1: int, d2: int) -> torch.Tensor:
+def get_kron_order(d1: int, d2: int) -> Tensor:
     r"""Get order for doing kronecker product."""
-    kron_order = torch.zeros([d1 * d2, 2], dtype=torch.int64)
+    kron_order = zeros([d1 * d2, 2], dtype=torch.int64)
     for i in range(d1):
         for j in range(d2):
             kron_order[i * d2 + j, 0] = i
@@ -71,9 +73,9 @@ class MKDGradients(nn.Module):
 
         self.grad = SpatialGradient(mode='diff', order=1, normalized=False)
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
-        if not isinstance(x, torch.Tensor):
-            raise TypeError(f"Input type is not a torch.Tensor. Got {type(x)}")
+    def forward(self, x: Tensor) -> Tensor:
+        if not isinstance(x, Tensor):
+            raise TypeError(f"Input type is not a Tensor. Got {type(x)}")
         if not len(x.shape) == 4:
             raise ValueError(f"Invalid input shape, we expect Bx1xHxW. Got: {x.shape}")
         # Modify 'diff' gradient. Before we had lambda function, but it is not jittable
@@ -110,23 +112,23 @@ class VonMisesKernel(nn.Module):
         >>> emb = vm(oris) # 23x7x32x32
     """
 
-    def __init__(self, patch_size: int, coeffs: Union[list, tuple]) -> None:
+    def __init__(self, patch_size: int, coeffs: Union[List[Union[float, int]], Tuple[Union[float, int], ...]]) -> None:
         super().__init__()
 
         self.patch_size = patch_size
-        b_coeffs: torch.Tensor = torch.tensor(coeffs)
+        b_coeffs = tensor(coeffs)
         self.register_buffer('coeffs', b_coeffs)
 
         # Compute parameters.
-        n: int = len(coeffs) - 1
-        self.n: int = n
-        self.d: int = 2 * n + 1
+        n = len(coeffs) - 1
+        self.n = n
+        self.d = 2 * n + 1
 
         # Precompute helper variables.
         emb0 = torch.ones([1, 1, patch_size, patch_size])
         frange = torch.arange(n) + 1
         frange = frange.reshape(-1, 1, 1)
-        weights = torch.zeros([2 * n + 1])
+        weights = zeros([2 * n + 1])
         weights[: n + 1] = torch.sqrt(b_coeffs)
         weights[n + 1 :] = torch.sqrt(b_coeffs[1:])
         weights = weights.reshape(-1, 1, 1)
@@ -134,15 +136,15 @@ class VonMisesKernel(nn.Module):
         self.register_buffer('frange', frange)
         self.register_buffer('weights', weights)
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
-        if not isinstance(x, torch.Tensor):
-            raise TypeError(f"Input type is not a torch.Tensor. Got {type(x)}")
+    def forward(self, x: Tensor) -> Tensor:
+        if not isinstance(x, Tensor):
+            raise TypeError(f"Input type is not a Tensor. Got {type(x)}")
 
         if not len(x.shape) == 4 or x.shape[1] != 1:
             raise ValueError(f"Invalid input shape, we expect Bx1xHxW. Got: {x.shape}")
 
         # TODO: unify the two lines below when pytorch 1.6 support is dropped
-        emb0: torch.Tensor = torch.jit.annotate(torch.Tensor, self.emb0)
+        emb0: Tensor = torch.jit.annotate(Tensor, self.emb0)
         emb0 = emb0.to(x).repeat(x.size(0), 1, 1, 1)
         frange = self.frange.to(x) * x
         emb1 = torch.cos(frange)
@@ -205,14 +207,14 @@ class EmbedGradients(nn.Module):
         _, phi = cart2pol(kgrid[:, :, :, 0], kgrid[:, :, :, 1])
         self.register_buffer('phi', phi)
 
-    def emb_mags(self, mags: torch.Tensor) -> torch.Tensor:
+    def emb_mags(self, mags: Tensor) -> Tensor:
         """Embed square roots of magnitudes with eps for numerical reasons."""
         mags = torch.sqrt(mags + self.eps)
         return mags
 
-    def forward(self, grads: torch.Tensor) -> torch.Tensor:
-        if not isinstance(grads, torch.Tensor):
-            raise TypeError(f"Input type is not a torch.Tensor. Got {type(grads)}")
+    def forward(self, grads: Tensor) -> Tensor:
+        if not isinstance(grads, Tensor):
+            raise TypeError(f"Input type is not a Tensor. Got {type(grads)}")
         if not len(grads.shape) == 4:
             raise ValueError(f"Invalid input shape, we expect Bx2xHxW. Got: {grads.shape}")
         mags = grads[:, :1, :, :]
@@ -235,7 +237,7 @@ class EmbedGradients(nn.Module):
         )
 
 
-def spatial_kernel_embedding(kernel_type, grids: dict) -> torch.Tensor:
+def spatial_kernel_embedding(kernel_type, grids: Dict[str, Tensor]) -> Tensor:
     r"""Compute embeddings for cartesian and polar parametrizations."""
     factors = {"phi": 1.0, "rho": pi / sqrt2, "x": pi / 2, "y": pi / 2}
     if kernel_type == 'cart':
@@ -333,25 +335,25 @@ class ExplicitSpacialEncoding(nn.Module):
         self.register_buffer('emb2', emb2)
         self.register_buffer('idx1', idx1)
 
-    def get_gmask(self, sigma: float) -> torch.Tensor:
+    def get_gmask(self, sigma: float) -> Tensor:
         """Compute Gaussian mask."""
         norm_rho = self.grid['rho'] / self.grid['rho'].max()
         gmask = torch.exp(-1 * norm_rho**2 / sigma**2)
         return gmask
 
-    def init_kron(self) -> Tuple[torch.Tensor, torch.Tensor]:
+    def init_kron(self) -> Tuple[Tensor, Tensor]:
         """Initialize helper variables to calculate kronecker."""
         kron = get_kron_order(self.in_dims, self.d_emb)
-        _emb = torch.jit.annotate(torch.Tensor, self.emb)
+        _emb = torch.jit.annotate(Tensor, self.emb)
         emb2 = torch.index_select(_emb, 1, kron[:, 1])
         return emb2, kron[:, 0]
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
-        if not isinstance(x, torch.Tensor):
-            raise TypeError(f"Input type is not a torch.Tensor. Got {type(x)}")
+    def forward(self, x: Tensor) -> Tensor:
+        if not isinstance(x, Tensor):
+            raise TypeError(f"Input type is not a Tensor. Got {type(x)}")
         if not ((len(x.shape) == 4) | (x.shape[1] == self.in_dims)):
             raise ValueError(f"Invalid input shape, we expect Bx{self.in_dims}xHxW. Got: {x.shape}")
-        idx1 = torch.jit.annotate(torch.Tensor, self.idx1)
+        idx1 = torch.jit.annotate(Tensor, self.idx1)
         emb1 = torch.index_select(x, 1, idx1)
         output = emb1 * self.emb2
         output = output.sum(dim=(2, 3))
@@ -392,7 +394,7 @@ class Whitening(nn.Module):
 
     Args:
         xform: Variant of whitening to use. None, 'lw', 'pca', 'pcaws', 'pcawt'.
-        whitening_model: Dictionary with keys 'mean', 'eigvecs', 'eigvals' holding torch.Tensors.
+        whitening_model: Dictionary with keys 'mean', 'eigvecs', 'eigvals' holding Tensors.
         in_dims: Dimensionality of input descriptors.
         output_dims: (int) Dimensionality reduction.
         keval: Shrinkage parameter.
@@ -422,7 +424,7 @@ class Whitening(nn.Module):
     def __init__(
         self,
         xform: str,
-        whitening_model: Union[Dict[str, Dict[str, torch.Tensor]], None],
+        whitening_model: Union[Dict[str, Dict[str, Tensor]], None],
         in_dims: int,
         output_dims: int = 128,
         keval: int = 40,
@@ -441,14 +443,14 @@ class Whitening(nn.Module):
         self.output_dims = output_dims
 
         # Initialize identity transform.
-        self.mean = nn.Parameter(torch.zeros(in_dims), requires_grad=True)
+        self.mean = nn.Parameter(zeros(in_dims), requires_grad=True)
         self.evecs = nn.Parameter(torch.eye(in_dims)[:, :output_dims], requires_grad=True)
         self.evals = nn.Parameter(torch.ones(in_dims)[:output_dims], requires_grad=True)
 
         if whitening_model is not None:
             self.load_whitening_parameters(whitening_model)
 
-    def load_whitening_parameters(self, whitening_model: Dict[str, Dict[str, torch.Tensor]]) -> None:
+    def load_whitening_parameters(self, whitening_model: Dict[str, Dict[str, Tensor]]) -> None:
         algo = 'lw' if self.xform == 'lw' else 'pca'
         wh_model = whitening_model[algo]
         self.mean.data = wh_model['mean']
@@ -483,9 +485,9 @@ class Whitening(nn.Module):
         m = -0.5 * self.t
         self.evecs.data = self.evecs @ torch.diag(torch.pow(self.evals, m))
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
-        if not isinstance(x, torch.Tensor):
-            raise TypeError(f"Input type is not a torch.Tensor. Got {type(x)}")
+    def forward(self, x: Tensor) -> Tensor:
+        if not isinstance(x, Tensor):
+            raise TypeError(f"Input type is not a Tensor. Got {type(x)}")
         if not len(x.shape) == 2:
             raise ValueError(f"Invalid input shape, we expect NxD. Got: {x.shape}")
         x = x - self.mean  # Center the data.
@@ -579,8 +581,9 @@ class MKDDescriptor(nn.Module):
 
         # Load supervised(lw)/unsupervised(pca) model trained on training_set.
         if self.whitening is not None:
-            storage_fcn: Callable = lambda storage, loc: storage
-            whitening_models = torch.hub.load_state_dict_from_url(urls[self.kernel_type], map_location=storage_fcn)
+            whitening_models = torch.hub.load_state_dict_from_url(
+                urls[self.kernel_type], map_location=map_location_to_cpu
+            )
             whitening_model = whitening_models[training_set]
             self.whitening_layer = Whitening(
                 whitening, whitening_model, in_dims=self.odims, output_dims=self.output_dims
@@ -588,9 +591,9 @@ class MKDDescriptor(nn.Module):
             self.odims = self.output_dims
         self.eval()
 
-    def forward(self, patches: torch.Tensor) -> torch.Tensor:
-        if not isinstance(patches, torch.Tensor):
-            raise TypeError(f"Input type is not a torch.Tensor. Got {type(patches)}")
+    def forward(self, patches: Tensor) -> Tensor:
+        if not isinstance(patches, Tensor):
+            raise TypeError(f"Input type is not a Tensor. Got {type(patches)}")
         if not len(patches.shape) == 4:
             raise ValueError(f"Invalid input shape, we expect Bx1xHxW. Got: {patches.shape}")
         # Extract gradients.
@@ -637,9 +640,8 @@ class MKDDescriptor(nn.Module):
         )
 
 
-def load_whitening_model(kernel_type: str, training_set: str) -> Dict:
-    storage_fcn: Callable = lambda storage, loc: storage
-    whitening_models = torch.hub.load_state_dict_from_url(urls[kernel_type], map_location=storage_fcn)
+def load_whitening_model(kernel_type: str, training_set: str) -> Dict[str, Any]:
+    whitening_models = torch.hub.load_state_dict_from_url(urls[kernel_type], map_location=map_location_to_cpu)
     whitening_model = whitening_models[training_set]
     return whitening_model
 
@@ -671,5 +673,5 @@ class SimpleKD(nn.Module):
 
         self.features = nn.Sequential(smoothing, gradients, ori, ese, wh)
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
+    def forward(self, x: Tensor) -> Tensor:
         return self.features(x)
