@@ -1,9 +1,9 @@
 from typing import Tuple, Union
 
 import torch
-import torch.nn as nn
 import torch.nn.functional as F
 
+from kornia.core import Module, Tensor, concatenate, stack, tensor, zeros
 from kornia.filters.sobel import spatial_gradient3d
 from kornia.geometry.conversions import normalize_pixel_coordinates, normalize_pixel_coordinates3d
 from kornia.utils import create_meshgrid, create_meshgrid3d
@@ -14,7 +14,7 @@ from .dsnt import spatial_expectation2d, spatial_softmax2d
 from .nms import nms3d
 
 
-def _get_window_grid_kernel2d(h: int, w: int, device: torch.device = torch.device('cpu')) -> torch.Tensor:
+def _get_window_grid_kernel2d(h: int, w: int, device: torch.device = torch.device('cpu')) -> Tensor:
     r"""Helper function, which generates a kernel to with window coordinates, residual to window center.
 
     Args:
@@ -31,7 +31,7 @@ def _get_window_grid_kernel2d(h: int, w: int, device: torch.device = torch.devic
     return conv_kernel
 
 
-def _get_center_kernel2d(h: int, w: int, device: torch.device = torch.device('cpu')) -> torch.Tensor:
+def _get_center_kernel2d(h: int, w: int, device: torch.device = torch.device('cpu')) -> Tensor:
     r"""Helper function, which generates a kernel to return center coordinates, when applied with F.conv2d to 2d
     coordinates grid.
 
@@ -43,7 +43,7 @@ def _get_center_kernel2d(h: int, w: int, device: torch.device = torch.device('cp
     Returns:
         conv_kernel [2x2xhxw].
     """
-    center_kernel = torch.zeros(2, 2, h, w, device=device)
+    center_kernel = zeros(2, 2, h, w, device=device)
 
     #  If the size is odd, we have one pixel for center, if even - 2
     if h % 2 != 0:
@@ -62,7 +62,7 @@ def _get_center_kernel2d(h: int, w: int, device: torch.device = torch.device('cp
     return center_kernel
 
 
-def _get_center_kernel3d(d: int, h: int, w: int, device: torch.device = torch.device('cpu')) -> torch.Tensor:
+def _get_center_kernel3d(d: int, h: int, w: int, device: torch.device = torch.device('cpu')) -> Tensor:
     r"""Helper function, which generates a kernel to return center coordinates, when applied with F.conv2d to 3d
     coordinates grid.
 
@@ -75,7 +75,7 @@ def _get_center_kernel3d(d: int, h: int, w: int, device: torch.device = torch.de
     Returns:
         conv_kernel [3x3xdxhxw].
     """
-    center_kernel = torch.zeros(3, 3, d, h, w, device=device)
+    center_kernel = zeros(3, 3, d, h, w, device=device)
     #  If the size is odd, we have one pixel for center, if even - 2
     if h % 2 != 0:
         h_i1 = h // 2
@@ -100,7 +100,7 @@ def _get_center_kernel3d(d: int, h: int, w: int, device: torch.device = torch.de
     return center_kernel
 
 
-def _get_window_grid_kernel3d(d: int, h: int, w: int, device: torch.device = torch.device('cpu')) -> torch.Tensor:
+def _get_window_grid_kernel3d(d: int, h: int, w: int, device: torch.device = torch.device('cpu')) -> Tensor:
     r"""Helper function, which generates a kernel to return coordinates, residual to window center.
 
     Args:
@@ -116,13 +116,13 @@ def _get_window_grid_kernel3d(d: int, h: int, w: int, device: torch.device = tor
     if d > 1:
         z = torch.linspace(-1, 1, d, device=device).view(d, 1, 1, 1)
     else:  # only onr channel with index == 0
-        z = torch.zeros(1, 1, 1, 1, device=device)
-    grid3d = torch.cat([z.repeat(1, h, w, 1).contiguous(), grid2d.repeat(d, 1, 1, 1)], dim=3)
+        z = zeros(1, 1, 1, 1, device=device)
+    grid3d = concatenate([z.repeat(1, h, w, 1).contiguous(), grid2d.repeat(d, 1, 1, 1)], 3)
     conv_kernel = grid3d.permute(3, 0, 1, 2).unsqueeze(1)
     return conv_kernel
 
 
-class ConvSoftArgmax2d(nn.Module):
+class ConvSoftArgmax2d(Module):
     r"""Module that calculates soft argmax 2d per window.
 
     See :func:`~kornia.geometry.subpix.conv_soft_argmax2d` for details.
@@ -133,7 +133,7 @@ class ConvSoftArgmax2d(nn.Module):
         kernel_size: Tuple[int, int] = (3, 3),
         stride: Tuple[int, int] = (1, 1),
         padding: Tuple[int, int] = (1, 1),
-        temperature: Union[torch.Tensor, float] = torch.tensor(1.0),
+        temperature: Union[Tensor, float] = tensor(1.0),
         normalized_coordinates: bool = True,
         eps: float = 1e-8,
         output_value: bool = False,
@@ -174,7 +174,7 @@ class ConvSoftArgmax2d(nn.Module):
             + ')'
         )
 
-    def forward(self, x: torch.Tensor):  # type: ignore
+    def forward(self, x: Tensor):
         return conv_soft_argmax2d(
             x,
             self.kernel_size,
@@ -187,7 +187,7 @@ class ConvSoftArgmax2d(nn.Module):
         )
 
 
-class ConvSoftArgmax3d(nn.Module):
+class ConvSoftArgmax3d(Module):
     r"""Module that calculates soft argmax 3d per window.
 
     See :func:`~kornia.geometry.subpix.conv_soft_argmax3d` for details.
@@ -198,7 +198,7 @@ class ConvSoftArgmax3d(nn.Module):
         kernel_size: Tuple[int, int, int] = (3, 3, 3),
         stride: Tuple[int, int, int] = (1, 1, 1),
         padding: Tuple[int, int, int] = (1, 1, 1),
-        temperature: Union[torch.Tensor, float] = torch.tensor(1.0),
+        temperature: Union[Tensor, float] = tensor(1.0),
         normalized_coordinates: bool = False,
         eps: float = 1e-8,
         output_value: bool = True,
@@ -245,7 +245,7 @@ class ConvSoftArgmax3d(nn.Module):
             + ')'
         )
 
-    def forward(self, x: torch.Tensor):  # type: ignore
+    def forward(self, x: Tensor):
         return conv_soft_argmax3d(
             x,
             self.kernel_size,
@@ -260,15 +260,15 @@ class ConvSoftArgmax3d(nn.Module):
 
 
 def conv_soft_argmax2d(
-    input: torch.Tensor,
+    input: Tensor,
     kernel_size: Tuple[int, int] = (3, 3),
     stride: Tuple[int, int] = (1, 1),
     padding: Tuple[int, int] = (1, 1),
-    temperature: Union[torch.Tensor, float] = torch.tensor(1.0),
+    temperature: Union[Tensor, float] = tensor(1.0),
     normalized_coordinates: bool = True,
     eps: float = 1e-8,
     output_value: bool = False,
-) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
+) -> Union[Tensor, Tuple[Tensor, Tensor]]:
     r"""Compute the convolutional spatial Soft-Argmax 2D over the windows of a given heatmap.
 
     .. math::
@@ -310,7 +310,7 @@ def conv_soft_argmax2d(
         >>> nms_coords, nms_val = conv_soft_argmax2d(input, (3,3), (2,2), (1,1), output_value=True)
     """
     if not torch.is_tensor(input):
-        raise TypeError(f"Input type is not a torch.Tensor. Got {type(input)}")
+        raise TypeError(f"Input type is not a Tensor. Got {type(input)}")
 
     if not len(input.shape) == 4:
         raise ValueError(f"Invalid input shape, we expect BxCxHxW. Got: {input.shape}")
@@ -324,8 +324,8 @@ def conv_soft_argmax2d(
     dtype: torch.dtype = input.dtype
     input = input.view(b * c, 1, h, w)
 
-    center_kernel: torch.Tensor = _get_center_kernel2d(kx, ky, device).to(dtype)
-    window_kernel: torch.Tensor = _get_window_grid_kernel2d(kx, ky, device).to(dtype)
+    center_kernel: Tensor = _get_center_kernel2d(kx, ky, device).to(dtype)
+    window_kernel: Tensor = _get_window_grid_kernel2d(kx, ky, device).to(dtype)
 
     # applies exponential normalization trick
     # https://timvieira.github.io/blog/post/2014/02/11/exp-normalize-trick/
@@ -347,13 +347,13 @@ def conv_soft_argmax2d(
 
     # We need to output also coordinates
     # Pooled window center coordinates
-    grid_global: torch.Tensor = create_meshgrid(h, w, False, device).to(dtype).permute(0, 3, 1, 2)
+    grid_global: Tensor = create_meshgrid(h, w, False, device).to(dtype).permute(0, 3, 1, 2)
 
     grid_global_pooled = F.conv2d(grid_global, center_kernel, stride=stride, padding=padding)
 
     # Coordinates of maxima residual to window center
     # prepare kernel
-    coords_max: torch.Tensor = F.conv2d(x_exp, window_kernel, stride=stride, padding=padding)
+    coords_max: Tensor = F.conv2d(x_exp, window_kernel, stride=stride, padding=padding)
 
     coords_max = coords_max / den.expand_as(coords_max)
     coords_max = coords_max + grid_global_pooled.expand_as(coords_max)
@@ -373,16 +373,16 @@ def conv_soft_argmax2d(
 
 
 def conv_soft_argmax3d(
-    input: torch.Tensor,
+    input: Tensor,
     kernel_size: Tuple[int, int, int] = (3, 3, 3),
     stride: Tuple[int, int, int] = (1, 1, 1),
     padding: Tuple[int, int, int] = (1, 1, 1),
-    temperature: Union[torch.Tensor, float] = torch.tensor(1.0),
+    temperature: Union[Tensor, float] = tensor(1.0),
     normalized_coordinates: bool = False,
     eps: float = 1e-8,
     output_value: bool = True,
     strict_maxima_bonus: float = 0.0,
-) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
+) -> Union[Tensor, Tuple[Tensor, Tensor]]:
     r"""Compute the convolutional spatial Soft-Argmax 3D over the windows of a given heatmap.
 
     .. math::
@@ -430,7 +430,7 @@ def conv_soft_argmax3d(
         >>> nms_coords, nms_val = conv_soft_argmax3d(input, (3, 3, 3), (1, 2, 2), (0, 1, 1))
     """
     if not torch.is_tensor(input):
-        raise TypeError(f"Input type is not a torch.Tensor. Got {type(input)}")
+        raise TypeError(f"Input type is not a Tensor. Got {type(input)}")
 
     if not len(input.shape) == 5:
         raise ValueError(f"Invalid input shape, we expect BxCxDxHxW. Got: {input.shape}")
@@ -444,8 +444,8 @@ def conv_soft_argmax3d(
     dtype: torch.dtype = input.dtype
     input = input.view(b * c, 1, d, h, w)
 
-    center_kernel: torch.Tensor = _get_center_kernel3d(kx, ky, kz, device).to(dtype)
-    window_kernel: torch.Tensor = _get_window_grid_kernel3d(kx, ky, kz, device).to(dtype)
+    center_kernel: Tensor = _get_center_kernel3d(kx, ky, kz, device).to(dtype)
+    window_kernel: Tensor = _get_window_grid_kernel3d(kx, ky, kz, device).to(dtype)
 
     # applies exponential normalization trick
     # https://timvieira.github.io/blog/post/2014/02/11/exp-normalize-trick/
@@ -462,13 +462,13 @@ def conv_soft_argmax3d(
 
     # We need to output also coordinates
     # Pooled window center coordinates
-    grid_global: torch.Tensor = create_meshgrid3d(d, h, w, False, device=device).to(dtype).permute(0, 4, 1, 2, 3)
+    grid_global: Tensor = create_meshgrid3d(d, h, w, False, device=device).to(dtype).permute(0, 4, 1, 2, 3)
 
     grid_global_pooled = F.conv3d(grid_global, center_kernel, stride=stride, padding=padding)
 
     # Coordinates of maxima residual to window center
     # prepare kernel
-    coords_max: torch.Tensor = F.conv3d(x_exp, window_kernel, stride=stride, padding=padding)
+    coords_max: Tensor = F.conv3d(x_exp, window_kernel, stride=stride, padding=padding)
 
     coords_max = coords_max / den.expand_as(coords_max)
     coords_max = coords_max + grid_global_pooled.expand_as(coords_max)
@@ -493,7 +493,7 @@ def conv_soft_argmax3d(
         in_levels: int = input.size(2)
         out_levels: int = x_softmaxpool.size(2)
         skip_levels: int = (in_levels - out_levels) // 2
-        strict_maxima: torch.Tensor = F.avg_pool3d(nms3d(input, kernel_size), 1, stride, 0)
+        strict_maxima: Tensor = F.avg_pool3d(nms3d(input, kernel_size), 1, stride, 0)
         strict_maxima = strict_maxima[:, :, skip_levels : out_levels - skip_levels]
         x_softmaxpool *= 1.0 + strict_maxima_bonus * strict_maxima
     x_softmaxpool = x_softmaxpool.view(b, c, x_softmaxpool.size(2), x_softmaxpool.size(3), x_softmaxpool.size(4))
@@ -501,8 +501,8 @@ def conv_soft_argmax3d(
 
 
 def spatial_soft_argmax2d(
-    input: torch.Tensor, temperature: torch.Tensor = torch.tensor(1.0), normalized_coordinates: bool = True
-) -> torch.Tensor:
+    input: Tensor, temperature: Tensor = tensor(1.0), normalized_coordinates: bool = True
+) -> Tensor:
     r"""Compute the Spatial Soft-Argmax 2D of a given input heatmap.
 
     Args:
@@ -523,20 +523,20 @@ def spatial_soft_argmax2d(
         >>> spatial_soft_argmax2d(input, normalized_coordinates=False)
         tensor([[[1.0000, 1.0000]]])
     """
-    input_soft: torch.Tensor = spatial_softmax2d(input, temperature)
-    output: torch.Tensor = spatial_expectation2d(input_soft, normalized_coordinates)
+    input_soft: Tensor = spatial_softmax2d(input, temperature)
+    output: Tensor = spatial_expectation2d(input_soft, normalized_coordinates)
     return output
 
 
-class SpatialSoftArgmax2d(nn.Module):
+class SpatialSoftArgmax2d(Module):
     r"""Compute the Spatial Soft-Argmax 2D of a given heatmap.
 
     See :func:`~kornia.geometry.subpix.spatial_soft_argmax2d` for details.
     """
 
-    def __init__(self, temperature: torch.Tensor = torch.tensor(1.0), normalized_coordinates: bool = True) -> None:
+    def __init__(self, temperature: Tensor = tensor(1.0), normalized_coordinates: bool = True) -> None:
         super().__init__()
-        self.temperature: torch.Tensor = temperature
+        self.temperature: Tensor = temperature
         self.normalized_coordinates: bool = normalized_coordinates
 
     def __repr__(self) -> str:
@@ -550,13 +550,11 @@ class SpatialSoftArgmax2d(nn.Module):
             + ')'
         )
 
-    def forward(self, input: torch.Tensor) -> torch.Tensor:
+    def forward(self, input: Tensor) -> Tensor:
         return spatial_soft_argmax2d(input, self.temperature, self.normalized_coordinates)
 
 
-def conv_quad_interp3d(
-    input: torch.Tensor, strict_maxima_bonus: float = 10.0, eps: float = 1e-7
-) -> Tuple[torch.Tensor, torch.Tensor]:
+def conv_quad_interp3d(input: Tensor, strict_maxima_bonus: float = 10.0, eps: float = 1e-7) -> Tuple[Tensor, Tensor]:
     r"""Compute the single iteration of quadratic interpolation of the extremum (max or min).
 
     Args:
@@ -588,20 +586,20 @@ def conv_quad_interp3d(
         >>> nms_coords, nms_val = conv_quad_interp3d(input, 1.0)
     """
     if not torch.is_tensor(input):
-        raise TypeError(f"Input type is not a torch.Tensor. Got {type(input)}")
+        raise TypeError(f"Input type is not a Tensor. Got {type(input)}")
 
     if not len(input.shape) == 5:
         raise ValueError(f"Invalid input shape, we expect BxCxDxHxW. Got: {input.shape}")
 
     B, CH, D, H, W = input.shape
-    grid_global: torch.Tensor = create_meshgrid3d(D, H, W, False, device=input.device).permute(0, 4, 1, 2, 3)
+    grid_global: Tensor = create_meshgrid3d(D, H, W, False, device=input.device).permute(0, 4, 1, 2, 3)
     grid_global = grid_global.to(input.dtype)
 
     # to determine the location we are solving system of linear equations Ax = b, where b is 1st order gradient
     # and A is Hessian matrix
-    b: torch.Tensor = spatial_gradient3d(input, order=1, mode='diff')  #
+    b: Tensor = spatial_gradient3d(input, order=1, mode='diff')  #
     b = b.permute(0, 1, 3, 4, 5, 2).reshape(-1, 3, 1)
-    A: torch.Tensor = spatial_gradient3d(input, order=2, mode='diff')
+    A: Tensor = spatial_gradient3d(input, order=2, mode='diff')
     A = A.permute(0, 1, 3, 4, 5, 2).reshape(-1, 6)
     dxx = A[..., 0]
     dyy = A[..., 1]
@@ -610,37 +608,37 @@ def conv_quad_interp3d(
     dys = 0.25 * A[..., 4]  # normalization to match OpenCV implementation
     dxs = 0.25 * A[..., 5]  # normalization to match OpenCV implementation
 
-    Hes = torch.stack([dxx, dxy, dxs, dxy, dyy, dys, dxs, dys, dss], dim=-1).view(-1, 3, 3)
+    Hes = stack([dxx, dxy, dxs, dxy, dyy, dys, dxs, dys, dss], -1).view(-1, 3, 3)
     if not torch_version_geq(1, 10):
         # The following is needed to avoid singular cases
         Hes += torch.rand(Hes[0].size(), device=Hes.device).abs()[None] * eps
 
-    nms_mask: torch.Tensor = nms3d(input, (3, 3, 3), True)
-    x_solved: torch.Tensor = torch.zeros_like(b)
+    nms_mask: Tensor = nms3d(input, (3, 3, 3), True)
+    x_solved: Tensor = torch.zeros_like(b)
     x_solved_masked, _, solved_correctly = safe_solve_with_mask(b[nms_mask.view(-1)], Hes[nms_mask.view(-1)])
 
     #  Kill those points, where we cannot solve
     new_nms_mask = nms_mask.masked_scatter(nms_mask, solved_correctly)
 
     x_solved.masked_scatter_(new_nms_mask.view(-1, 1, 1), x_solved_masked[solved_correctly])
-    dx: torch.Tensor = -x_solved
+    dx: Tensor = -x_solved
 
     # Ignore ones, which are far from window center
     mask1 = dx.abs().max(dim=1, keepdim=True)[0] > 0.7
     dx.masked_fill_(mask1.expand_as(dx), 0)
-    dy: torch.Tensor = 0.5 * torch.bmm(b.permute(0, 2, 1), dx)
+    dy: Tensor = 0.5 * torch.bmm(b.permute(0, 2, 1), dx)
     y_max = input + dy.view(B, CH, D, H, W)
     if strict_maxima_bonus > 0:
         y_max += strict_maxima_bonus * new_nms_mask.to(input.dtype)
 
-    dx_res: torch.Tensor = dx.flip(1).reshape(B, CH, D, H, W, 3).permute(0, 1, 5, 2, 3, 4)
-    coords_max: torch.Tensor = grid_global.repeat(B, 1, 1, 1, 1).unsqueeze(1)
+    dx_res: Tensor = dx.flip(1).reshape(B, CH, D, H, W, 3).permute(0, 1, 5, 2, 3, 4)
+    coords_max: Tensor = grid_global.repeat(B, 1, 1, 1, 1).unsqueeze(1)
     coords_max = coords_max + dx_res
 
     return coords_max, y_max
 
 
-class ConvQuadInterp3d(nn.Module):
+class ConvQuadInterp3d(Module):
     r"""Calculate soft argmax 3d per window.
 
     See :func:`~kornia.geometry.subpix.conv_quad_interp3d` for details.
@@ -655,5 +653,5 @@ class ConvQuadInterp3d(nn.Module):
     def __repr__(self) -> str:
         return self.__class__.__name__ + '(' + 'strict_maxima_bonus=' + str(self.strict_maxima_bonus) + ')'
 
-    def forward(self, x: torch.Tensor):  # type: ignore
+    def forward(self, x: Tensor):
         return conv_quad_interp3d(x, self.strict_maxima_bonus, self.eps)
